#!/bin/bash

# $Id:  $
# Copyright (c) 201e NetApp, Inc.
# All rights reserved.

PKGNAME=vpupgrader

MAINT_USER=maint
MAINT_GROUP=maintenance
MAINT_HOME="/home/${MAINT_USER}"
MAINT_SCRIPTS="${MAINT_HOME}/scripts"
LOG_DIR="/var/log"
PRODUCT_NAME="SnapCenter Plug-in for VMware vSphere"

upgradelog=$1
cmdoutlog="/var/log/upgrade-cmdout"
restart=false
post_reboot_mysql_upgrade=false
post_reboot_rabbitmq_upgrade=false

cstat=0
function eval_it()
{
    eval "$@" &> ${cmdoutlog}
    cstat="$?"
    cat ${cmdoutlog} | tee -a ${upgradelog}
}

function echo_log()
{
    tstamp=`date +'%b %d %T'`
    echo "$@"
    echo "${tstamp}:  $@" >> ${upgradelog}
}

function log()
{
    tstamp=`date +'%b %d %T'`
    echo "${tstamp}:  $@" &>> ${upgradelog}
}

function vercomp(){
    local IFS=.
    local i first=($1) second=($2)
    for ((i=${#first[@]}; i<${#second[@]}; i++))
    do
        first[i]=0

    done
    for ((i=0; i<${#first[@]}; i++))
    do
        if [[ -z ${second[i]} ]]
        then
            second[i]=0

        fi
        if ((10#${first[i]} > 10#${second[i]}))
        then
            verresult="1"
            return 0
        fi
        if ((10#${first[i]} < 10#${second[i]}))
        then
            verresult="2"
            return 0
        fi
    done
    return 0
}

# Need to update the initrd when the kernel headers change.
function updateVMwareTools()
{
    log "Updating VMware Tools for new kernel header files"
    log "`/usr/bin/vmware-config-tools.pl -d default`"
}

function UpdateKernel() {
    # Get the current installed version
    installedKernelVersion=$(uname -r | awk -F'-amd64' '{print $1}')
    log "installedKernelVersion=$installedKernelVersion"
    echo_log "installedKernelVersion=$installedKernelVersion"
    installedKernalVersionForCompare="${installedKernelVersion/-/.}"
    log "installedKernalVersionForCompare=$installedKernalVersionForCompare"
    echo_log "installedKernalVersionForCompare=$installedKernalVersionForCompare"

    # IN: linux-headers-4.19.0-17-amd64 - Header files for Linux 4.19.0-17-amd64, OUT: 4.19.0-17
    isoKernelVersion=$(apt-cache search --names-only 'linux-headers-.*-amd64' | awk '{print $1}' | sort | head -n 1 | sed -e 's/linux-headers-\(.*\)-amd64/\1/')
    log "isoKernelVersion=${isoKernelVersion}"
    echo_log "isoKernelVersion=${isoKernelVersion}"
    isoKernelVersionForCompare="$(echo ${isoKernelVersion} | sed "s/-/./")"
    log "isoKernelVersionForCompare=${isoKernelVersionForCompare}"
    echo_log "isoKernelVersionForCompare=${isoKernelVersionForCompare}"

    # IN: linux-kbuild-4.19 - Kbuild infrastructure for Linux 4.19, OUT: linux-kbuild-4.19
    kbuildPackage=$(apt-cache search --names-only 'linux-kbuild-.*' | awk '{print $1}' | sort | head -n 1)
    log "kbuildPackage=${kbuildPackage}"
    echo_log "kbuildPackage=${kbuildPackage}"

    verresult=0
    vercomp "${installedKernalVersionForCompare}" "${isoKernelVersionForCompare}"
    echo "vercompresult=$verresult"
    echo_log "vercompresult=$verresult"
    if [[ ${verresult} -eq 2 ]]; then
        log "New kernel available. Will upgrade Kernel and reboot the vApp"
        DEBIAN_FRONTEND=noninteractive apt-get -qq -y --force-yes install linux-image-${isoKernelVersion}-amd64  >> ${upgradelog}
	echo_log "After install linux-image.... $?"
        DEBIAN_FRONTEND=noninteractive apt-get -qq -y --force-yes install linux-headers-${isoKernelVersion}-amd64 >> ${upgradelog}
	echo_log "After install linux-headers.... $?"
        DEBIAN_FRONTEND=noninteractive apt-get -qq -y --force-yes install linux-headers-${isoKernelVersion}-common  >> ${upgradelog}
	echo_log "After install linux-headers-common.... $?"
        DEBIAN_FRONTEND=noninteractive apt-get -qq -y --force-yes install ${kbuildPackage} >> ${upgradelog}
	echo_log "After install linux-kbuild.... $?"
        updateVMwareTools
	echo_log "After updateVMWareTools.... $?"

        # The grub.cfg is based on the OVA fresh install... we need to change some files because of some hardcoding
        filenames=`ls /etc/grub.d/*`
        for filename in $filenames; do
            sed -i 's/\/boot\/vmlinuz.*amd64/\/vmlinuz/g' $filename
            sed -i 's/\/boot\/initrd.img.*amd64/\/initrd.img/g' $filename
        done

        # Force the old eth0 naming for now in order to keep ova and iso upgrade consistent.
   	sed -i '/^linux \/vmlinuz/s/$/ net.ifnames=0/' /etc/grub.d/50_platform
    	update-grub2 &>> ${upgradelog}

        restart="true"
    fi
}

#     ************* MAIN ***************
function main() {
    chattr -i /etc/passwd /etc/shadow
    echo_log "Upgrading ${PRODUCT_NAME} OS and applications..."

    echo_log "Stopping MySQL , NGINX and RabbitMQ servers ..."
    /usr/sbin/service mysql stop
	/etc/init.d/nginx stop
    service rabbitmq-server stop
    echo_log "Stopped MySQL, NGINX and RabbitMQ servers"

    echo 'Acquire::cdrom::AutoDetect "false";' > /etc/apt/apt.conf
    echo_log 'Dpkg::Options { "--force-confold"; "--force-confdef"; }' >> /etc/apt/apt.conf

    echo "Before Starting update....." >> ${upgradelog}
    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -qq update  --allow-unauthenticated >> ${upgradelog}'
    echo "After Starting update.....$?" >> ${upgradelog}
    if [[ ${cstat} != 0 ]]
    then
        exit 1
    fi

    echo_log "Check for kernel upgrade ..."
    # Upgrade the kernel first if there is any kernel upgrade required
    UpdateKernel

    echo "Before Starting dist-upgrade....." >> ${upgradelog}
    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --force-yes --allow-unauthenticated install openssh-server >> ${upgradelog}'
    echo_log "After openssh-server...$?"

    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --force-yes --allow-unauthenticated install debconf-utils >> ${upgradelog}'
    echo_log "After debconf-utils...$?"

cat <<EOL | debconf-set-selections
grub-pc grub-pc/install_devices multiselect /dev/sda
grub-pc grub-pc/install_devices_empty boolean false
EOL

    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get upgrade -y --force-yes --allow-unauthenticated >> ${upgradelog}'
    echo_log "After upgrade..."
    if [[ ${cstat} != 0 ]]
    then
      echo_log "upgrade script failed"
      exit 1
    fi

    chattr -i /etc/passwd /etc/shadow
    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get dist-upgrade -y -f --force-yes --allow-unauthenticated >> ${upgradelog} '
    echo_log "After dist-upgrade..... $?"

    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt --fix-broken install -y --force-yes --allow-unauthenticated  >> ${upgradelog}'
    echo_log "After fix-broken... $?"

    eval_it 'DEBIAN_FRONTEND=noninteractive /usr/bin/apt-get -y --allow-remove-essential --allow-unauthenticated install initramfs-tools >> ${upgradelog}'
    echo_log "After initramfs-tools...$?"

    eval_it 'DEBIAN_FRONTEND=noninteractive apt --fix-broken install -y --force-yes --allow-unauthenticated  >> ${upgradelog}'
    echo_log "After fix-broken... $?"
    if [[ ${cstat} != 0 ]]
    then
        exit 1
    fi

    apt -y --allow-unauthenticated autoremove
    echo_log "After autoremove....$?"
    # Run any initialization/firstboot scripts that were added as part of the upgrade.
    #/usr/sbin/service vplatform start

    # Update appliance upgrade log
    # and version file.
    if [ ! -f /APPLIANCE_UPGRADE ] ; then
        cat <<EOF > /APPLIANCE_UPGRADE
###
### Original Install
###

EOF
        cat /APPLIANCE_BUILD >> /APPLIANCE_UPGRADE
    fi
    chattr -i /APPLIANCE_BUILD /APPLIANCE_UPGRADE
    chmod 0666 /APPLIANCE_BUILD /APPLIANCE_UPGRADE
    cat <<EOF >> /APPLIANCE_UPGRADE
###
### Upgrade $(date +'%b %d %T')
###

EOF

    cat /upgrade/APPLIANCE_BUILD >> /APPLIANCE_UPGRADE
    cp /upgrade/APPLIANCE_BUILD /APPLIANCE_BUILD
    chmod 0444 /APPLIANCE_BUILD /APPLIANCE_UPGRADE
    chattr +i /APPLIANCE_BUILD /APPLIANCE_UPGRADE

    # Restart the system if changes necessitate
    # These steps might not be required for now, but we may need them in future.
    if [ "${restart}" == "true" ]; then
        echo_log "Changes during ${PRODUCT_NAME} upgrade necessitates a"
        echo_log "restart of this Virtual Machine..."

        /sbin/shutdown -r now
    fi

    # Start mysql ,nginx and rabbitmq services
    /usr/sbin/service mysql start
	/etc/init.d/nginx start
    service rabbitmq-server restart

    # Run any initialization/firstboot scripts that were added as part of the upgrade.
    /usr/sbin/service vplatform start
    exit 0
}

main "$@"
